//=================================================================
//                   CopyrightUpdater Class
//=================================================================
//    This class provides a simple graphical interface to update
// the copyright information in a source Java file. The essential
// functionality is a line pattern match replacement scheme.
//
//                << CopyrightUpdater.java >>
//
//=================================================================
// Copyright (C) 2013 Dana Proctor
// Version 1.4 03/14/2013
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version
// 2 of the License, or (at your option) any later version. This
// program is distributed in the hope that it will be useful, 
// but WITHOUT ANY WARRANTY; without even the implied warranty
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
// the GNU General Public License for more details. You should
// have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
// (http://opensource.org)
//
//=================================================================
// Revision History
// Changes to the code should be documented here and reflected
// in the present version number. Author information should
// also be included with the original copyright author.
//=================================================================
// Version 1.0 01/18/2013 Initial CopyrightUpdater Class.
//         1.1 01/23/2013 Working Functional Class.
//         1.2 01/24/2013 Added Header Replacement Feature Instead
//                        of Just Line Update.
//         1.3 01/25/2013 Made Found Pattern for Update Set to 
//                        updateCopyrightTxtField Also. Minor FindBugs Fixes
//                        & Border Formatting Label and Panels.
//         1.4 03/14/2013 Update Package to Reflect Dandy Made Productions.
//
//-----------------------------------------------------------------
//                 danap@dandymadeproductions.com
//=================================================================

package com.dandymadeproductions.copyrightupdater;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.text.DefaultEditorKit;

/**
 *    This class provides a simple graphical interface to update the
 * copyright information in a source Java file. The essential functionality
 * is a line pattern match replacement scheme.
 * 
 * @author Dana M. Proctor
 * @version 1.4 03/14/2013
 */

public class CopyrightUpdater extends JFrame implements ActionListener
{
   // Class Instances
   private static final long serialVersionUID = 8289679233299381990L;

   private JPanel updateReplacementPanel;
   private CardLayout modeCardLayout;

   private JRadioButton updateRadioButton, replaceRadioButton;
   private JButton sourceDirectoryButton;
   private JTextField startsWithTxtField, containsTxtField_1, containsTxtField_2;
   private JTextField copyrightLineTxtField, updateCopyrightTxtField;
   private JTextArea replaceCopyrightHeaderTxtArea;
   private JLabel statusLabel;
   private JTextArea messageTextArea;
   private JButton updateReplaceButton;

   private String sourceDirectory, lastDirectory;
   private boolean modeUpdate;

   private static final String JAVA_EXT = ".java";
   private static final String JAVA_IN_EXT = ".java.in";

   private static final String ACTION_UPDATE = "Update";
   private static final String ACTION_REPLACE = "Replace";

   //==============================================================
   // CopyrightUpdater Constructor (Just All the GUI Stuff)
   //==============================================================

   public CopyrightUpdater(String title)
   {
      super(title);

      // Constructor Instances
      JTabbedPane mainTabsPane;
      JPanel mainPanel, inputPanel, outputPanel;
      JPanel modePanel, sourcePanel;
      JPanel updatePanel, replacementPanel;
      JPanel modeSourceSelectorPanel, optionsSelectorPanel;
      JPanel copyrightPanel, updateCopyrightPanel;
      JPanel actionPanel;

      JLabel sourceSelectorLabel;
      JLabel lineStartsWithLabel, lineContainsLabel, lineAndLabel;
      JLabel copyrightLabel, updateLabel;
      JLabel replacementLabel;

      // Setup
      sourceDirectory = "";
      lastDirectory = "";
      modeUpdate = true;

      // Main panel
      mainPanel = new JPanel(new BorderLayout());

      mainTabsPane = new JTabbedPane();
      mainTabsPane.setBorder(BorderFactory.createLoweredBevelBorder());
      mainTabsPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

      // Input panel Elements

      inputPanel = new JPanel();
      inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));

      //=============================================
      // Mode & Source directory selector components.

      modeSourceSelectorPanel = new JPanel(new GridLayout(1, 2));
      modeSourceSelectorPanel.setBorder(BorderFactory.createRaisedBevelBorder());

      // Mode
      modePanel = new JPanel();
      modePanel.setBorder(BorderFactory.createEtchedBorder());

      ButtonGroup updateReplaceButtonGroup = new ButtonGroup();

      updateRadioButton = new JRadioButton(ACTION_UPDATE, true);
      updateRadioButton.setFocusPainted(false);
      updateRadioButton.setActionCommand(ACTION_UPDATE);
      updateRadioButton.addActionListener(this);
      updateReplaceButtonGroup.add(updateRadioButton);
      modePanel.add(updateRadioButton);

      replaceRadioButton = new JRadioButton(ACTION_REPLACE, true);
      replaceRadioButton.setFocusPainted(false);
      replaceRadioButton.setActionCommand(ACTION_REPLACE);
      replaceRadioButton.addActionListener(this);
      updateReplaceButtonGroup.add(replaceRadioButton);
      modePanel.add(replaceRadioButton);

      modeSourceSelectorPanel.add(modePanel);

      // Source Directory
      sourcePanel = new JPanel();
      sourcePanel.setBorder(BorderFactory.createEtchedBorder());

      ImageIcon openIcon = new ImageIcon("images/icons/openIcon.png");
      sourceDirectoryButton = new JButton(openIcon);
      sourceDirectoryButton.setHorizontalAlignment(JButton.CENTER);
      sourceDirectoryButton.setMargin(new Insets(0, 0, 0, 0));
      sourceDirectoryButton.addActionListener(this);
      sourcePanel.add(sourceDirectoryButton);

      sourceSelectorLabel = new JLabel("Select Source Directory");
      sourcePanel.add(sourceSelectorLabel);

      modeSourceSelectorPanel.add(sourcePanel);

      inputPanel.add(modeSourceSelectorPanel);

      //=============================================
      // Update/Replacement Panels

      modeCardLayout = new CardLayout();
      updateReplacementPanel = new JPanel(modeCardLayout);

      // UPDATE
      // Line Replacement Options
      updatePanel = new JPanel(new GridLayout(3, 1));

      optionsSelectorPanel = new JPanel();
      optionsSelectorPanel.setBorder(
         BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(),
                                            BorderFactory.createEmptyBorder(5, 2, 5, 2)));

      lineStartsWithLabel = new JLabel("Line Starts With: ");
      optionsSelectorPanel.add(lineStartsWithLabel);

      startsWithTxtField = new JTextField(10);
      startsWithTxtField.setBorder(BorderFactory.createLoweredBevelBorder());
      startsWithTxtField.setText("*");
      addComponentPopup(startsWithTxtField);
      optionsSelectorPanel.add(startsWithTxtField);

      lineContainsLabel = new JLabel("Line Contains: ");
      optionsSelectorPanel.add(lineContainsLabel);

      containsTxtField_1 = new JTextField(25);
      containsTxtField_1.setBorder(BorderFactory.createLoweredBevelBorder());
      containsTxtField_1.setText("Copyright (c)");
      addComponentPopup(containsTxtField_1);
      optionsSelectorPanel.add(containsTxtField_1);
      
      lineAndLabel = new JLabel("And");
      optionsSelectorPanel.add(lineAndLabel);
      
      containsTxtField_2 = new JTextField(25);
      containsTxtField_2.setBorder(BorderFactory.createLoweredBevelBorder());
      containsTxtField_2.setText("PostgreSQL");
      addComponentPopup(containsTxtField_2);
      optionsSelectorPanel.add(containsTxtField_2);
      
      updatePanel.add(optionsSelectorPanel);

      // Identified copyright source line components.
      copyrightPanel = new JPanel();
      copyrightPanel.setLayout(new BoxLayout(copyrightPanel, BoxLayout.Y_AXIS));
      copyrightPanel.setBorder(
         BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(),
                                            BorderFactory.createEmptyBorder(5, 2, 5, 2)));

      copyrightLabel = new JLabel("Located Source Copyright Line", JLabel.LEFT);
      copyrightLabel.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
      copyrightPanel.add(copyrightLabel);

      copyrightLineTxtField = new JTextField(72);
      copyrightLineTxtField.setEnabled(false);
      copyrightLineTxtField.setBorder(BorderFactory.createLoweredBevelBorder());
      addComponentPopup(copyrightLineTxtField);

      copyrightPanel.add(new JScrollPane(copyrightLineTxtField));

      updatePanel.add(copyrightPanel);

      // Update line content replacement components.
      updateCopyrightPanel = new JPanel();
      updateCopyrightPanel.setLayout(new BoxLayout(updateCopyrightPanel, BoxLayout.Y_AXIS));
      updateCopyrightPanel.setBorder(BorderFactory.createEtchedBorder());

      updateLabel = new JLabel("Replacement Copyright Line", JLabel.LEFT);
      updateLabel.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
      updateCopyrightPanel.add(updateLabel);

      updateCopyrightTxtField = new JTextField(72);
      updateCopyrightTxtField.setEnabled(false);
      updateCopyrightTxtField.setBorder(BorderFactory.createLoweredBevelBorder());
      addComponentPopup(updateCopyrightTxtField);

      updateCopyrightPanel.add(new JScrollPane(updateCopyrightTxtField));

      updatePanel.add(updateCopyrightPanel);

      updateReplacementPanel.add(ACTION_UPDATE, updatePanel);

      // REPLACEMENT
      // Header Replacement Entry

      replacementPanel = new JPanel();
      replacementPanel.setBorder(
         BorderFactory.createCompoundBorder(BorderFactory.createEtchedBorder(),
                                            BorderFactory.createEmptyBorder(5, 2, 5, 2)));
      replacementPanel.setLayout(new BoxLayout(replacementPanel, BoxLayout.Y_AXIS));

      replacementLabel = new JLabel("Replacement Header", JLabel.CENTER);
      replacementLabel.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 0));
      replacementPanel.add(replacementLabel);

      replaceCopyrightHeaderTxtArea = new JTextArea();
      replaceCopyrightHeaderTxtArea.setBorder(BorderFactory.createLoweredBevelBorder());
      addComponentPopup(replaceCopyrightHeaderTxtArea);
      replacementPanel.add(new JScrollPane(replaceCopyrightHeaderTxtArea));

      updateReplacementPanel.add(ACTION_REPLACE, replacementPanel);

      inputPanel.add(updateReplacementPanel);

      // Status Label & Proceed with Update Button
      actionPanel = new JPanel();
      actionPanel.setBorder(BorderFactory.createRaisedBevelBorder());

      updateReplaceButton = new JButton("Update");
      updateReplaceButton.setEnabled(false);
      updateReplaceButton.addActionListener(this);
      actionPanel.add(updateReplaceButton);

      statusLabel = new JLabel("Status: Waiting");
      statusLabel.setBorder(BorderFactory.createEmptyBorder(0, 10, 0, 10));
      actionPanel.add(statusLabel);

      inputPanel.add(actionPanel);

      mainTabsPane.addTab("Processing", inputPanel);

      //======================
      // Output panel elements

      outputPanel = new JPanel(new GridLayout(1, 1));

      messageTextArea = new JTextArea();
      addComponentPopup(messageTextArea);
      outputPanel.add(new JScrollPane(messageTextArea));

      mainTabsPane.addTab("Messages", outputPanel);

      mainPanel.add(mainTabsPane, BorderLayout.CENTER);

      // Add to frame.
      mainPanel.setOpaque(true);
      setContentPane(mainPanel);
   }

   //==============================================================
   // Method to add a popup menu listener for the cut, copy, and
   // paste for components.
   //==============================================================

   private void addComponentPopup(JComponent component)
   {
      component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, KeyEvent.CTRL_MASK),
         DefaultEditorKit.copyAction);
      component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, KeyEvent.SHIFT_MASK),
         DefaultEditorKit.cutAction);
      component.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_INSERT, KeyEvent.SHIFT_MASK),
         DefaultEditorKit.pasteAction);

      component.addMouseListener(new InternalPopup(component));
   }

   //==============================================================
   // ActionEvent Listener method for detecting the user's actions.
   //==============================================================

   public void actionPerformed(ActionEvent evt)
   {
      Object panelSource = evt.getSource();

      // Mode selection copyright line update or header replacement.

      if (panelSource instanceof JRadioButton)
      {
         String actionCommand = ((JRadioButton) panelSource).getActionCommand();

         if (actionCommand.equals(ACTION_UPDATE))
         {
            modeUpdate = true;
            modeCardLayout.show(updateReplacementPanel, ACTION_UPDATE);

            if (updateReplaceButton.isEnabled() && !copyrightLineTxtField.isEnabled())
            {
               statusLabel.setText("Status: Pattern Not Determined, Select Source Directory Again.");
               updateReplaceButton.setEnabled(false);
            }
         }
         else
         {
            modeUpdate = false;
            modeCardLayout.show(updateReplacementPanel, ACTION_REPLACE);
         }
      }

      // This is where the actions are to function calls.

      if (panelSource instanceof JButton)
      {
         // Collect the source directory and thereby
         // also setting up the initial locate and
         // replacement line information.

         if (((JButton) panelSource) == sourceDirectoryButton)
         {
            sourceDirectory = selectSourceDirectory();
            messageTextArea.setText("Source Directory: " + sourceDirectory + "\n");

            if (!sourceDirectory.isEmpty())
            {
               // Line Replacement
               if (modeUpdate)
               {
                  statusLabel.setText("Processing Source Directory");
                  collectProcessInformation(sourceDirectory);
               }
               // Header Replacement
               else
               {
                  updateReplaceButton.setEnabled(true);
               }

               // Messaging
               if (!updateReplaceButton.isEnabled() && modeUpdate)
                  statusLabel.setText("Failed to Locate Java Source File or Line Pattern!");
               else
               {
                  if (modeUpdate)
                     statusLabel.setText("Ready: Enter Update Text");
                  else
                     statusLabel.setText("Ready: Enter Replacement Header");
               }
            }
            else
               statusLabel.setText("Source Directory Empty!");
         }

         // Update the copyright information.

         else if (((JButton) panelSource) == updateReplaceButton)
         {
            if (!modeUpdate && replaceCopyrightHeaderTxtArea.getText().isEmpty())
            {
               int results = JOptionPane.showConfirmDialog(null,
                  "Warning No Header Data Entered for Replacement!", "Alert", JOptionPane.OK_CANCEL_OPTION);
               if (results == JOptionPane.CANCEL_OPTION)
                  return;
            }

            updateCopyright(new File(sourceDirectory));
            statusLabel.setText("Status: Replacement Complete");
         }
         return;
      }
   }

   //==============================================================
   // Class Method to collect the source directory to be processed.
   //==============================================================

   private String selectSourceDirectory()
   {
      // Method Instances.
      JFileChooser directoryChooser;
      String directoryName, fileSeparator;

      // Choosing the directory to import data from.
      if (lastDirectory.equals(""))
         directoryChooser = new JFileChooser();
      else
         directoryChooser = new JFileChooser(new File(lastDirectory));

      directoryChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
      int result = directoryChooser.showOpenDialog(this);

      // Looks like might be good so lets check and read data.
      if (result == JFileChooser.APPROVE_OPTION)
      {
         // Save the selected directory so can be used again.
         lastDirectory = directoryChooser.getCurrentDirectory().toString();

         // Collect the directory.
         fileSeparator = System.getProperty("file.separator");

         if (fileSeparator == null || fileSeparator.equals(""))
            fileSeparator = "/";

         directoryName = lastDirectory + fileSeparator + directoryChooser.getSelectedFile().getName();

         // Return the source Directory.
         if (!directoryName.equals(""))
            return directoryName;
         else
         {
            JOptionPane.showMessageDialog(null, "File NOT Found!", "Alert", JOptionPane.ERROR_MESSAGE);
            return "";
         }
      }
      return "";
   }

   //==============================================================
   // Class method to find an inital source file to collect info
   // for beginning the processing. All we want to do here is read
   // the first java source code file found and locate startwith
   // characters and contains string.
   // 
   // This will only be used when doing a line update in source.
   //==============================================================

   private void collectProcessInformation(String fileNameString)
   {
      // Method Instances
      File srcDirectoryFile, subDirectory, validJavaFile;
      File[] directoryFiles;
      boolean foundDirectory;

      srcDirectoryFile = new File(fileNameString);
      subDirectory = null;
      validJavaFile = null;
      foundDirectory = false;

      if (srcDirectoryFile.isDirectory())
      {
         directoryFiles = srcDirectoryFile.listFiles();

         for (int i = 0; i < directoryFiles.length; i++)
         {
            if (directoryFiles[i].isDirectory() && !foundDirectory)
               subDirectory = directoryFiles[i];

            if (directoryFiles[i].isFile() && directoryFiles[i].getName().endsWith(JAVA_EXT))
            {
               validJavaFile = directoryFiles[i];
               break;
            }
         }
         if (validJavaFile == null && subDirectory != null)
            collectProcessInformation(subDirectory.getAbsolutePath());

         // Looks a valid Java source file was found
         // so read it to locate line to replace to
         // verify update.

         if (validJavaFile != null)
         {
            FileReader fileReader;
            BufferedReader bufferedReader;
            int maxLinesToRead = 100;

            copyrightLineTxtField.setText("");
            updateCopyrightTxtField.setText("");
            copyrightLineTxtField.setEnabled(false);
            updateCopyrightTxtField.setEnabled(false);
            updateReplaceButton.setEnabled(false);

            messageTextArea.append("Found Java File: " + validJavaFile + "\n");

            fileReader = null;
            bufferedReader = null;

            try
            {
               fileReader = new FileReader(validJavaFile);
               bufferedReader = new BufferedReader(fileReader);
               String currentLine;

               int lineNumber = 1;

               while ((currentLine = bufferedReader.readLine()) != null && lineNumber < maxLinesToRead)
               {
                  if (currentLine.startsWith(startsWithTxtField.getText().trim())
                      && currentLine.toLowerCase().indexOf(containsTxtField_1.getText().toLowerCase()) != -1
                      && currentLine.toLowerCase().indexOf(containsTxtField_2.getText().toLowerCase()) != -1)
                  {
                     copyrightLineTxtField.setText(currentLine);
                     copyrightLineTxtField.setEnabled(true);
                     
                     updateCopyrightTxtField.setText(currentLine);
                     updateCopyrightTxtField.setEnabled(true);
                     updateReplaceButton.setEnabled(true);
                     break;
                  }
               }
            }
            catch (IOException e)
            {
               JOptionPane.showMessageDialog(null, "Unable to Process File! \n" + validJavaFile.getName(),
                  "Alert", JOptionPane.ERROR_MESSAGE);
            }
            finally
            {
               try
               {
                  if (bufferedReader != null)
                     bufferedReader.close();
               }
               catch (IOException ioe)
               {
                  messageTextArea.append("CollectProcessInformation() " + "Failed to Close BufferedReader\n");
               }
               finally
               {
                  try
                  {
                     if (fileReader != null)
                        fileReader.close();
                  }
                  catch (IOException ioe2)
                  {
                     messageTextArea.append("CollectProcessInformation() " + "Failed to Close FileReader\n");
                  }
               }
            }
         }
      }
   }

   //==============================================================
   // Class method to actually perform the work of updating the
   // copyright line with the replacement pattern or completely
   // replacing the header portion of the source.
   //
   // Note:
   // This routine by being theaded could dramatically increase
   // the performance of replacements in large subdirectory
   // trees structures.
   //==============================================================

   private void updateCopyright(File srcDirectory)
   {
      // Method Instances
      File[] directoryFiles;

      FileReader fileReader;
      BufferedReader bufferedReader;
      FileWriter fileWriter;
      BufferedWriter bufferedWriter;

      StringBuffer fileContentBuffer;
      String currentLine;

      boolean readFileFailure, patternFindFailure;
      boolean packageLineReached;

      // Setup
      fileContentBuffer = new StringBuffer();

      // Process directory.

      directoryFiles = srcDirectory.listFiles();

      for (int i = 0; i < directoryFiles.length; i++)
      {
         readFileFailure = false;
         patternFindFailure = true;
         packageLineReached = false;
         fileContentBuffer.delete(0, fileContentBuffer.length());

         // Transverse recursively
         if (directoryFiles[i].isDirectory())
            updateCopyright(directoryFiles[i]);

         // Process Files.
         else
         {
            if (directoryFiles[i].getName().endsWith(JAVA_EXT)
                || directoryFiles[i].getName().endsWith(JAVA_IN_EXT))
            {
               statusLabel.setText("Processing: " + directoryFiles[i].getName());
               messageTextArea.append("Processing: " + directoryFiles[i].getName() + "\n");

               // Read the source file and create a buffer of its
               // data to include the new replacement pattern/header.

               fileReader = null;
               bufferedReader = null;

               try
               {
                  fileReader = new FileReader(directoryFiles[i]);
                  bufferedReader = new BufferedReader(fileReader);

                  if (!modeUpdate)
                     fileContentBuffer.append(replaceCopyrightHeaderTxtArea.getText() + "\n");

                  while ((currentLine = bufferedReader.readLine()) != null)
                  {
                     // Update Line in Source.
                     if (modeUpdate)
                     {
                        // Append replacement pattern.
                        if (currentLine.startsWith(startsWithTxtField.getText().trim())
                            && currentLine.toLowerCase().indexOf(containsTxtField_1.getText().toLowerCase()) != -1
                            && currentLine.toLowerCase().indexOf(containsTxtField_2.getText().toLowerCase()) != -1)
                        {
                           fileContentBuffer.append(updateCopyrightTxtField.getText() + "\n");
                           patternFindFailure = false;
                        }
                        else
                           fileContentBuffer.append(currentLine + "\n");
                     }
                     // Relace Header in Source.
                     else
                     {
                        if (packageLineReached)
                           fileContentBuffer.append(currentLine + "\n");
                        else
                        {
                           // This may not be very good for no package and
                           // imports declarations.
                           if (currentLine.indexOf("package") != -1 || currentLine.indexOf("import") != -1
                               || currentLine.indexOf("class") != -1)
                           {
                              packageLineReached = true;
                              fileContentBuffer.append(currentLine + "\n");
                           }
                        }
                     }
                  }
               }
               catch (IOException e)
               {
                  messageTextArea.append("updateCopyright() " + e.toString() + "\n");
                  readFileFailure = true;
               }
               finally
               {
                  try
                  {
                     if (bufferedReader != null)
                        bufferedReader.close();
                  }
                  catch (IOException ioe1)
                  {
                     messageTextArea.append("updateCopyright() " + ioe1.toString() + "\n");
                  }
                  finally
                  {
                     try
                     {
                        if (fileReader != null)
                           fileReader.close();
                     }
                     catch (IOException ioe2)
                     {
                        messageTextArea.append("updateCopyright() " + ioe2.toString() + "\n");
                     }
                  }
               }

               // Write the new buffered data with replacement
               // pattern/header back to the same file as read,
               // overwrites.

               // Check if pattern was even found.
               if (patternFindFailure && modeUpdate)
               {
                  messageTextArea.append(directoryFiles[i].getAbsolutePath() + " : Failed to Find Pattern\n");
                  continue;
               }

               if (!readFileFailure)
               {
                  fileWriter = null;
                  bufferedWriter = null;

                  try
                  {
                     fileWriter = new FileWriter(directoryFiles[i]);
                     bufferedWriter = new BufferedWriter(fileWriter);

                     bufferedWriter.write(fileContentBuffer.toString());
                     bufferedWriter.flush();
                     fileWriter.flush();
                  }
                  catch (Exception e)
                  {
                     messageTextArea.append("updateCopyright() " + e.toString() + "\n");
                     messageTextArea.append("Possibly Corrupt Source File: "
                                            + directoryFiles[i].getAbsolutePath() + "\n");
                  }
                  finally
                  {
                     try
                     {
                        if (bufferedWriter != null)
                           bufferedWriter.close();
                     }
                     catch (IOException ioe1)
                     {
                        messageTextArea.append("updateCopyright() " + ioe1.toString() + "\n");
                     }
                     finally
                     {
                        try
                        {
                           if (fileWriter != null)
                              fileWriter.close();
                        }
                        catch (IOException ioe2)
                        {
                           messageTextArea.append("updateCopyright() " + ioe2.toString() + "\n");
                        }
                     }
                  }
               }
            }
         }
      }
   }

   //==============================================================
   // Internal Class to Handle Component Popups, (Cut, Copy, Paste)
   //==============================================================

   private static class InternalPopup extends JPopupMenu implements MouseListener
   {
      private static final long serialVersionUID = -569514517636870505L;
      private JComponent popupComponent;

      // Constructor
      InternalPopup(JComponent popupComponent)
      {
         this.popupComponent = popupComponent;

         DefaultEditorKit kit = new DefaultEditorKit();

         add(createItem("Cut", DefaultEditorKit.cutAction, kit.getActions()));
         add(createItem("Copy", DefaultEditorKit.copyAction, kit.getActions()));
         add(createItem("Paste", DefaultEditorKit.pasteAction, kit.getActions()));
      }

      // Menu creation
      private JMenuItem createItem(String text, String key, Action[] actions)
      {
         JMenuItem item = new JMenuItem();

         for (int i = 0; i < actions.length; i++)
         {
            if (actions[i].getValue(Action.NAME).toString().equals(key))
            {
               item.setAction(actions[i]);
               item.setText(text);
            }
         }
         return item;
      }

      // Mouse intefaces
      public void mousePressed(MouseEvent me)
      {
         popupComponent.requestFocus();
      }

      public void mouseReleased(MouseEvent me)
      {
         if (SwingUtilities.isRightMouseButton(me))
         {
            show(popupComponent, me.getX(), me.getY());
         }
      }

      public void mouseClicked(MouseEvent me)
      {
      }

      public void mouseEntered(MouseEvent me)
      {
      }

      public void mouseExited(MouseEvent me)
      {
      }
   }

   //==============================================================
   // Main public access point method for instantiating the
   // application.
   //==============================================================

   public static void main(String[] args)
   {
      javax.swing.SwingUtilities.invokeLater(new Runnable()
      {
         public void run()
         {
            // Create and set up the window.

            CopyrightUpdater updaterFrame = new CopyrightUpdater("Copyright Updater");
            updaterFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

            // Display the window.

            updaterFrame.pack();
            updaterFrame.setVisible(true);
         }
      });
   }
}